import Mapper

# TAGS INDICATE IDENTIAL SHEETS
# NUMBERS are completely unconstrained
# LETTERS enforce internal symmetry
# < indicates a tile, with the arrow pointed in direction of polarization and feed
# X indicates an empty gap
# || or = (double lines) imply mirror symmetry on either side

#  |----------------------------------------------|
#  |  < |  1 |  < |  1 ||  < ||  1 |  < |  1 |  < |
#  |----------------------------------------------|
#  |  1 |  X |  2 |  X ||  A ||  X |  2 |  X |  1 |
#  |----------------------------------------------|
#  |  < |  2 |  < |  2 ||  < ||  2 |  < |  2 |  < |
#  |----------------------------------------------|
#  |  1 |  X |  2 |  X ||  A ||  X |  2 |  X |  1 |
#  |==============================================|
#  |  < |  B |  < |  B ||  < ||  B |  < |  B |  < |
#  |==============================================|
#  |  1 |  X |  2 |  X ||  A ||  X |  2 |  X |  1 |
#  |----------------------------------------------|
#  |  < |  2 |  < |  2 ||  < ||  2 |  < |  2 |  < |
#  |----------------------------------------------|
#  |  1 |  X |  2 |  X ||  A ||  X |  2 |  X |  1 |
#  |----------------------------------------------|
#  |  < |  1 |  < |  1 ||  < ||  1 |  < |  1 |  < |
#  |----------------------------------------------|

#     Symmetric about the E and H plane axis, Identical edges and interiors
#              TOTAL NUMBER OF BITS = 76

# maps the switch states so that the array is symetric about the E plane
# bit ordeer is left to right, top to bottom by switch and then by sheet
class SymmetricInteriorVsEdge(Mapper.Mapper):
    # intializes the mapper
    # index map is a dictionary
    # return type: self
    def __init__(self):
        indexMap = {}
        bit_idx = 0
        # assign first edge sheet new bits (1)
        for switch_y in range(3,-4,-1):
            for switch_x in range(-3,4,1):
                num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                if num_switch_steps != 0:
                    coordinates = (-3,4,switch_x,switch_y)
                    indexMap[coordinates] = bit_idx
                    bit_idx += 1

        # duplicate edge to H edges in first quadrant
        for sheet_y in [4]:
            for sheet_x in range(-4,0,1):
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_y in range(3,-4,-1):
                        for switch_x in range(-3,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                duplicate_coordinates = (-3,4, switch_x, switch_y)
                                bit_loc = indexMap[duplicate_coordinates]
                                indexMap[coordinates] = bit_loc

        # duplicate edge to E edges in first quadrant
        for sheet_y in range(4,0,-1):
            for sheet_x in [-4]:
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_y in range(3,-4,-1):
                        for switch_x in range(-3,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                duplicate_coordinates = (-3,4, switch_x, switch_y)
                                bit_loc = indexMap[duplicate_coordinates]
                                indexMap[coordinates] = bit_loc

        # assign first interior sheet new bits (2)
        for switch_y in range(3,-4,-1):
            for switch_x in range(-3,4,1):
                num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                if num_switch_steps != 0:
                    coordinates = (-2,3,switch_x,switch_y)
                    indexMap[coordinates] = bit_idx
                    bit_idx += 1

        # duplicate to other interior sheets in first quadrant
        for sheet_y in range(3,0,-1):
            for sheet_x in range(-3,0,1):
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_y in range(3,-4,-1):
                        for switch_x in range(-3,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                duplicate_coordinates = (-2,3,switch_x,switch_y)
                                bit_loc = indexMap[duplicate_coordinates]
                                indexMap[coordinates] = bit_loc

        # assign left half of first sheet in center colomn new bits (A)
        for switch_y in range(3,-4,-1):
            for switch_x in range(-3,1,1):
                num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                if num_switch_steps != 0:
                    coordinates = (0,3,switch_x,switch_y)
                    indexMap[coordinates] = bit_idx
                    bit_idx += 1

        # duplicate left half of first sheet in center colomn to next sheet in center colomn
        for switch_y in range(3,-4,-1):
            for switch_x in range(-3,1,1):
                num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                if num_switch_steps != 0:
                    coordinates = (0,1,switch_x,switch_y)
                    duplicate_coordinates = (0,3,switch_x,switch_y)
                    bit_loc = indexMap[duplicate_coordinates]
                    indexMap[coordinates] = bit_loc

        # assign top half of first sheet in center row new bits (B)
        for switch_y in range(3,-1,-1):
            for switch_x in range(-3,4,1):
                num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                if num_switch_steps != 0:
                    coordinates = (-3,0,switch_x,switch_y)
                    indexMap[coordinates] = bit_idx
                    bit_idx += 1

        # duplicate top half of first sheet in center row to next sheet in center row
        for switch_y in range(3,-1,-1):
            for switch_x in range(-3,4,1):
                num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                if num_switch_steps != 0:
                    coordinates = (-1,0,switch_x,switch_y)
                    duplicate_coordinates = (-3,0,switch_x,switch_y)
                    bit_loc = indexMap[duplicate_coordinates]
                    indexMap[coordinates] = bit_loc

        # assign bottom half of center row (left array half) of sheets mirrored bits
        for sheet_y in [0]:
            for sheet_x in range(-4,0,1):
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_y in range(-1,-4,-1):
                        for switch_x in range(-3,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                mirror_coordinates = (sheet_x,sheet_y,switch_x,-1*switch_y)
                                bit_loc = indexMap[mirror_coordinates]
                                indexMap[coordinates] = bit_loc

        # assign right half of center colomn (top array half) of sheets mirrored bits
        for sheet_y in range(4,0,-1):
            for sheet_x in [0]:
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_y in range(3,-4,-1):
                        for switch_x in range(1,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                mirror_coordinates = (sheet_x,sheet_y,-1*switch_x,switch_y)
                                bit_loc = indexMap[mirror_coordinates]
                                indexMap[coordinates] = bit_loc


        # assign bottom left quadrant (included center column) mirrored bits
        for sheet_y in range(-1,-5,-1):
            for sheet_x in range(-4,1,1):
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_y in range(3,-4,-1):
                        for switch_x in range(-3,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                mirror_coordinates = (sheet_x,-1*sheet_y,switch_x,-1*switch_y)
                                bit_loc = indexMap[mirror_coordinates]
                                indexMap[coordinates] = bit_loc

        # assign right half of array (excluding center column) mirrored bits
        for sheet_y in range(4,-5,-1):
            for sheet_x in range(1,5,1):
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_y in range(3,-4,-1):
                        for switch_x in range(-3,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                mirror_coordinates = (-1*sheet_x,sheet_y,-1*switch_x,switch_y)
                                bit_loc = indexMap[mirror_coordinates]
                                indexMap[coordinates] = bit_loc

        # assign unused sheets to same bit (BIT 0)
        for sheet_y in [-5]:
            for sheet_x in range(-4,6):
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_x in range(-3,4,1):
                        for switch_y in range(-3,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                indexMap[coordinates] = 0
        for sheet_y in range(-4,6):
            for sheet_x in [-5]:
                num_sheet_steps = (abs(sheet_x)+abs(sheet_y))%2
                if num_sheet_steps != 0:
                    for switch_x in range(-3,4,1):
                        for switch_y in range(-3,4,1):
                            num_switch_steps = (abs(switch_x)+abs(switch_y))%2
                            if num_switch_steps != 0:
                                coordinates = (sheet_x,sheet_y,switch_x,switch_y)
                                indexMap[coordinates] = 0
        super(SymmetricInteriorVsEdge,self).__init__(76,indexMap)
